/*
 * DialogController.cpp
 *
 *  Created on: 5.1.2010
 *      Author: apoi
 */

#include <QList>
#include <QTimer>

#include "DialogController.h"
#include "BlogView.h"
#include "ErrorDialog.h"
#include "ProgressDialog.h"

DialogController::DialogController(BlogView *aParent, bool cancellable) : 
    QObject(aParent), iShowTaskProgress(false), cancellable(cancellable)
{
    iParent = aParent;
    iProgress = new ProgressDialog(aParent, this, cancellable);
    iError = new ErrorDialog(aParent, this);

    iCommandMessages << tr("Preparing")
            << tr("Fetching main page")
            << tr("Fetching blog RSD")
            << tr("Fetching user's blogs")
            << tr("Fetching pages")
            << tr("Fetching posts")
            << tr("Fetching categories")
            << tr("Fetching tags")
            << tr("Fetching page statuses")
            << tr("Fetching post statuses")
            << tr("Fetching comment statuses")
            << tr("Fetching comments")
            << tr("Loading a page")
            << tr("Adding a new page")
            << tr("Editing a page")
            << tr("Deleting a page")
            << tr("Loading a comment")
            << tr("Adding a new comment")
            << tr("Editing a comment")
            << tr("Deleting a comment")
            << tr("Loading a post")
            << tr("Adding a new post")
            << tr("Editing a post")
            << tr("Deleting a post")
            << tr("Fetching Gravatar")
            << tr("Adding a new category")
            << tr("Uploading a file")
            << tr("Fetching blog statistics")
            << tr("")
            << tr("Fetching location");
}

DialogController::~DialogController()
{
}
    
void DialogController::StartTaskProgress()
{
    // start display progress bar
    iShowTaskProgress = true;

    if (!iCommandList.isEmpty()) {
        qDebug("WP::DialogController::StartTaskProgress - Showing current task progress");

        // have active commands, show progress for currently executing task
        ShowTaskProgress();

    } else {
        qDebug("WP::DialogController::StartTaskProgress - Showing upcoming task progress");

    }
}

void DialogController::ShowInfoNote (const QString &title, const QString &message)
{
    if (iProgress->isVisible())
        iProgress->hide();

    iError->SetTitle(title);
    iError->ShowErrorNote(message);
    iError->MoveAndShow();
}
    
void DialogController::ShowErrorNote (const QString &error)
{
    ShowInfoNote(tr("Error occurred"), error);
}

void DialogController::TaskStarted (TWPNetworkEngineCommand command, bool foreground)
{
    // check status
	if (iCommandList.isEmpty()) {
        qDebug("WP::DialogController: Busy");

        // idle -> busy
	    iParent->ActivateBusyIndicator();
	}

    // record
    iCommandList.append(command);
	
    qDebug("WP::DialogController::TaskStarted %d: %d", command, iCommandList.size());

	// bring up progress dialog if needed:
    //  this task is a foreground task, or we are showing all tasks
    //  no progress or error dialog is already visible
    if ((foreground || iShowTaskProgress) && !(iProgress->isVisible() || iError->isVisible()))
        ShowTaskProgress();
}

void DialogController::TaskFinished (TWPNetworkEngineCommand command)
{
    // record
    iCommandList.removeOne(command);

    CheckStatus();

    qDebug("WP::DialogController::TaskFinished %d: %d", command, iCommandList.size());
}

void DialogController::CheckStatus(){
    // check status
    if (iCommandList.isEmpty()) {
        // busy -> idle
	    iParent->DeactivateBusyIndicator();

        // close progress dialog once idle
        // use a small hysteresis delay to avoid flickering
        QTimer::singleShot(100, this, SLOT(DelayedProgressDialogClose()));
    }
}
    
void DialogController::TaskFailed (TWPNetworkEngineCommand command, QString msg)
{
    // counts as a TaskFinished
    TaskFinished(command);
    
    // ignore progress
    if (iProgress->isVisible())
        iProgress->hide();

    // show error dialog
    iError->SetTitle(iCommandMessages.at(command));
    iError->ShowErrorNote(msg);
    iError->MoveAndShow();
}

void DialogController::ExitProgress()
{
    // XXX: cancel request instead
    if(cancellable){
        iCommandList.pop_front();
        CheckStatus();
        emit TaskCancelled();
    } else {
        iParent->Exit();
    }
}

void DialogController::ErrorAccepted()
{
    iError->hide();

    if (!iCommandList.isEmpty())
        // continue with progress
        iProgress->MoveAndShow();
    /*else
        // return to main
        iParent->SetActiveView();*/
}

void DialogController::ShowTaskProgress ()
{
    // set appropriate message based on iCommandMessages
    SetProgressMessage();

    // display
    iProgress->MoveAndShow();
}

void DialogController::SetProgressMessage()
{
    if (iCommandList.isEmpty())
        // XXX: shouldn't happen
        iProgress->SetAction(iCommandMessages.at(0));
    else
        // describe active request
        iProgress->SetAction(iCommandMessages.at(iCommandList.first()) + "...");
}

void DialogController::DelayedProgressDialogClose()
{
    if (iCommandList.isEmpty()) {
        // progress is done
        iProgress->hide();
        
        // unset temporal task progress once all tasks are over
        iShowTaskProgress = false;

        qDebug("WP::DialogController: Idle");

        /*if (!iError->isVisible())
            // return
            iParent->SetActiveView();*/

    } else {
        // update message to reflect command
        SetProgressMessage();
        iProgress->MoveAndShow();
    }
}

