/**
 * Implementations of Page-related functionality in WPData, both in WPDataPost and outside
 */

#include "WPData.h"

#include <QDebug>

const QString WPDataPost::ITEM_ID_KEY = "postid";

WPDataPost::WPDataPost (WPDataBlog &blog) :
    WPDataItem<WPDataPost>(blog, blog.iPosts)
{

}
    
WPDataPost::~WPDataPost()
{

}

void WPDataPost::GetContent (WPXmlItem *xml) const
{
    xml->content.insert("title",        iTitle);
    xml->content.insert("description",  iDescription);
    xml->content.insert("post_status",  iStatus);
    xml->content.insert("mt_keywords",  iMtKeywords);
    xml->content.insert("<dateTime.iso8601>date_created_gmt", GetISO8601Time());
    xml->content.insert("wp_password",  iWpPassword);
    
    xml->listContent.insert("categories", iCategories);

    if(iLocation.isValid()){
        xml->customFields.insert("geo_latitude", QString().setNum(iLocation.coordinate().latitude()));
        xml->customFields.insert("geo_longitude", QString().setNum(iLocation.coordinate().longitude()));
    }
}

void WPDataPost::SetContent (const WPXmlItem &xml)
{
    // typed fields
    SetISO8601Time(xml.content["date_created_gmt"]);

    // generic fields
	iItemId                 = xml.content[ITEM_ID_KEY];
	iStatus                 = xml.content["post_status"];

    // item fields
	iDescription            = xml.content["description"];
	iTitle                  = xml.content["title"];
	iLink                   = xml.content["link"];
	iPermaLink              = xml.content["permaLink"];
	iUserId                 = xml.content["userid"];
	iMtExcerpt              = xml.content["mt_excerpt"];
	iMtTextMore             = xml.content["mt_text_more"];
	iMtAllowComments        = xml.content["mt_allow_comments"];
	iMtAllowPings           = xml.content["mt_allow_pings"];
	iMtKeywords             = xml.content["mt_keywords"];
	iWpSlug                 = xml.content["wp_slug"];
	iWpPassword             = xml.content["wp_password"];
	iWpAuthorId             = xml.content["wp_author_id"];
	iWpAuthorDisplayName    = xml.content["wp_author_display_name"];

    iCategories             = xml.listContent["categories"];

    iCustomFields               = xml.customFields;
    
    if(iCustomFields.contains("geo_latitude") && iCustomFields.contains("geo_longitude")){
        double latitude = iCustomFields.value("geo_latitude").toDouble();
        double longitude = iCustomFields.value("geo_longitude").toDouble();
        
        iLocation = QtMobility::QGeoPositionInfo(QtMobility::QGeoCoordinate(latitude, longitude), QDateTime::currentDateTime());    
        
        qDebug(iLocation.coordinate().toString().toAscii().data());    
    }


    // fix data
    if (!iMtTextMore.isEmpty()) {
        // post is split into two parts
        iDescription = (
                iDescription
            +   "\n <!--more--> \n"
            +   iMtTextMore
        );
    }
    
    // updated from network, so we can't be dirty anymore
    iDirty = false;
    iError = false;
}

// XXX: move to WPDataItem?
void WPDataPost::AddFileUrl (const QString &url)
{
    // format HTML
    QString html = 
        "\n<p><a href=\"" + url + "\">" +
        "<img class=\"alignnone size-full wp-image-6\" src=\"" + url + "\" />" +
        "</a></p>\n";

    // append
    iDescription += html;
    
    // unsaved changes
    SetDirty();
}
    
void WPDataPost::ItemDeleted ()
{
    // react to remove this post's comments from list
    iBlog.iComments.RefreshAll();
}
    
QString WPDataPost::GetDisplayHTML ()
{
    // copy
    QString html(iDescription);

    html.replace("\n", "<br />");

    return html;
}
    
QList<WPComment> WPDataPost::GetComments ()
{
    QList<WPComment> comments;

    for (QListIterator<WPComment> i = iBlog.GetComments(); i.hasNext(); ) {
        WPComment comment = i.next();

        if (comment->iPostId == this->iItemId)
            comments.append(comment);
    }

    return comments;
}
    
WPComment WPDataPost::NewComment (const QString &body)
{
    // construct new comment and submit it
    return WPComment(new WPDataComment(iBlog, WPPost(this), body));
}

void WPDataPost::UpdateLocal (
            QString title, QString description, QString tags, QDateTime dateCreated,
            QString password, QStringList categories, const QtMobility::QGeoPositionInfo& location
) {
    // update fields
    iTitle = title;
    iDescription = description;
    iMtKeywords = tags;
    iDateCreated = dateCreated;
    iWpPassword = password;
    iCategories = categories;
    iLocation = QtMobility::QGeoPositionInfo(location);

    // super
    WPDataItem<WPDataPost>::UpdateLocal();
}

void WPDataPost::UpdateStatus (
        QString title, QString description, QString tags, QDateTime dateCreated,
        QString password, QStringList categories,
        QString status, QStringList addedFiles, const QtMobility::QGeoPositionInfo& location
) {
    // update fields
    iTitle = title;
    iDescription = description;
    iMtKeywords = tags;
    iDateCreated = dateCreated;
    iWpPassword = password;
    iCategories = categories;
    iLocation = QtMobility::QGeoPositionInfo(location);

    // super
    WPDataItem<WPDataPost>::UpdateStatus(status, addedFiles);
}

QDataStream &operator<< (QDataStream &out, const WPDataPost &post)
{
    out 
        << KDataPostVersionNumber
        << post.GetISO8601Time()
        << post.iDescription
        << post.iTitle
        << post.iLink
        << post.iPermaLink
        << post.iUserId
        << post.iItemId
        << post.iMtExcerpt
        << post.iMtTextMore
        << post.iMtAllowComments
        << post.iMtAllowPings
        << post.iMtKeywords
        << post.iWpSlug
        << post.iWpPassword
        << post.iWpAuthorId
        << post.iWpAuthorDisplayName
        << post.GetISO8601Time() // iDateCreatedGmt
        << post.iStatus
        << post.iLocation
    ;

    return out;
}

QDataStream &operator>> (QDataStream &in, WPDataPost &post)
{
    int version;
    QString strDate;
    QString unused;

    in 
        >> version
        >> strDate
        >> post.iDescription
        >> post.iTitle
        >> post.iLink
        >> post.iPermaLink
        >> post.iUserId
        >> post.iItemId
        >> post.iMtExcerpt
        >> post.iMtTextMore
        >> post.iMtAllowComments
        >> post.iMtAllowPings
        >> post.iMtKeywords
        >> post.iWpSlug
        >> post.iWpPassword
        >> post.iWpAuthorId
        >> post.iWpAuthorDisplayName
        >> unused // iDateCreatedGmt 
        >> post.iStatus;
 
    qDebug("Version: %i", version);
 
    if(version >= 2) 
        in >> post.iLocation;

    post.SetISO8601Time(strDate);

    return in;
}

