#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QLabel>

#include "commentsgraphicsview.h"
#include "wpcommentgraphicslistitem.h"
#include "wpcommentgraphicslist.h"
#include "WPData.h"
#include "commentsview.h"

#include "PopupMenu.h"
#include "themedatastorage.h"
#include "BlogView.h"
#include "wpcommongeometry.h"
#include "wpconfirmdialog.h"

WPCommentGraphicsList::WPCommentGraphicsList(QListIterator<WPComment> aData,
        BlogView* aBlogView, const QGraphicsView& aGraphicsView,
        PannableView* aPannableView, bool aShowFullComment
    ) : WPGraphicsList(aBlogView, aGraphicsView, aPannableView),
        iGravatars(aBlogView->GetGravatars()),
        iUnapproved(0),
        iApproved(0)
{
    // TODO find a proper way to align menu icon and text side by side.
    iActionApprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Approve"), this);
    iActionUnapprove = new QAction(iThemeData.iContextMenuIcons[ThemeData::EApprove],tr("    Unapprove"), this);
    iActionView = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    View"), this);
    iActionHide = new QAction(iThemeData.iContextMenuIcons[ThemeData::EView],tr("    Minimize"), this);
    iActionReply = new QAction(iThemeData.iContextMenuIcons[ThemeData::EReply],tr("    Reply"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);
    iActionSpam = new QAction(iThemeData.iContextMenuIcons[ThemeData::ESpam],tr("    Spam"), this);

    connect(iActionApprove,   SIGNAL(triggered()), this, SLOT(Approve()));
    connect(iActionUnapprove, SIGNAL(triggered()), this, SLOT(Unapprove()));
    connect(iActionView,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionHide,      SIGNAL(triggered()), this, SLOT(View()));
    connect(iActionReply,     SIGNAL(triggered()), this, SLOT(Reply()));
    connect(iActionDelete,    SIGNAL(triggered()), this, SLOT(Delete()));
    connect(iActionSpam,      SIGNAL(triggered()), this, SLOT(Spam()));

    connect(
        iGravatars, SIGNAL(GravatarUpdated (const QString &, const QPixmap &)),
        this,         SLOT(GravatarUpdated (const QString &, const QPixmap &))
    );    
    
    iUnapprovedLabel = new QLabel;
    iUnapprovedProxy = new QGraphicsProxyWidget(this);
    iUnapprovedProxy->setWidget(iUnapprovedLabel);
    iUnapprovedLabel->setObjectName(QString::fromLatin1("unapprovedLabel"));
    iUnapprovedLabel->setFixedHeight(KTitleDefaultHeight);
    iUnapprovedLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iApprovedLabel = new QLabel;
    iApprovedProxy = new QGraphicsProxyWidget(this);
    iApprovedProxy->setWidget(iApprovedLabel);
    iApprovedLabel->setObjectName(QString::fromLatin1("approvedLabel"));
    iApprovedLabel->setFixedHeight(KTitleDefaultHeight);
    iApprovedLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    // add initial comments, inserting labels as needed
    while (aData.hasNext()) {
        WPComment comment = aData.next();

        // sorted insert, with gravatar
        AddComment(comment, aShowFullComment);
    }

    // update label text, and container height
    SetLabelText();
}

WPCommentGraphicsList::~WPCommentGraphicsList()
{

}

void WPCommentGraphicsList::AddComment(WPComment aComment, bool aShowFullComment)
{
    WPCommentGraphicsListItem* item = new WPCommentGraphicsListItem(aComment, aShowFullComment, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(Selected(WPGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPGraphicsListItem*, QPoint)));

    // get gravatar for item
    item->SetAvatarImage(iGravatars->GetGravatar(aComment));

    // update counters/labels
    if (aComment->IsUnapproved())
        IncrementUnapproved();
    else
        IncrementApproved();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aComment->IsUnapproved() && index > 0)
                // unapproved comment at end of unapproved list, on the approved label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPComment list_comment = ItemAt(index)->GetComment();
        
        // unapproved go first
        if (aComment->IsUnapproved() && !list_comment->IsUnapproved())
            // XXX: never run, because of NULL index above
            break;

        if (!aComment->IsUnapproved() && list_comment->IsUnapproved())
            // do not mix approved in with unapproved comments
            continue;

        // find first older comment
        if (list_comment->GetLocalTime() < aComment->GetLocalTime())
            break;
    }

    // insert before older comment
    iItems.insert(index, item);
    iLayout->insertItem(index, item);
}

void WPCommentGraphicsList::RemoveComment (WPComment aComment)
{
    int index;
    WPCommentGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = ItemAt(index);

        if (item && item->GetComment() == aComment)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPComment->IsUnapproved(), because it might have changed
    bool isUnapproved = index <= iUnapproved;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isUnapproved)
        DecrementUnapproved();
    else
        DecrementApproved();
}

void WPCommentGraphicsList::GravatarUpdated (const QString &email, const QPixmap &pixmap)
{
    // look for ALL items with matching email and set their pixmap
    for (int i = 0; i < iItems.count(); i++) {
        WPCommentGraphicsListItem *item = ItemAt(i);

        // check for placeholders
        if (!item)
            continue;

        // normalize for comparison
        QString normalizedEmail = WPDataGravatars::NormalizeEmail(item->GetComment()->iAuthorEmail);

        if (normalizedEmail == email)
            // update item's pixmap
            item->SetAvatarImage(pixmap);
            
        // set avatar for each matching item
        continue;
    }
}

void WPCommentGraphicsList::PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu)
{
	if (CurrentItem()->GetComment()->IsUnapproved())
	{
		aPopupMenu->addAction(iActionApprove);
		aPopupMenu->setActiveAction(iActionApprove);
	}
	else
	{
		aPopupMenu->addAction(iActionUnapprove);
		aPopupMenu->setActiveAction(iActionUnapprove);
	}
	if (CurrentItem()->ShowFullComment())
		aPopupMenu->addAction(iActionHide);
	else
		aPopupMenu->addAction(iActionView);

	// XXX: if (iNetworkEngine.GetPost(aItem->GetComment().iPostId))
		aPopupMenu->addAction(iActionReply);

	aPopupMenu->addAction(iActionDelete);
	if (CurrentItem()->GetComment()->iStatus != QLatin1String("spam"))
		aPopupMenu->addAction(iActionSpam);
}

void WPCommentGraphicsList::Approve()
{
    CurrentItem()->GetComment()->UpdateStatus("approve");
}

void WPCommentGraphicsList::Unapprove()
{
    CurrentItem()->GetComment()->UpdateStatus("hold");
}

void WPCommentGraphicsList::View()
{
    CurrentItem()->ChangeViewMode();
    setMinimumHeight(iLayout->sizeHint(Qt::PreferredSize).height());
}

void WPCommentGraphicsList::Reply()
{
    emit ReplyToComment(CurrentItem()->GetComment());
}

void WPCommentGraphicsList::Delete()
{
    WP_CONFRIM_AND_RETURN_ON_FALSE;

    CurrentItem()->GetComment()->Delete();
}

void WPCommentGraphicsList::Spam()
{
   CurrentItem()->GetComment()->UpdateStatus("spam");
}

void WPCommentGraphicsList::CommentChanged(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentChanged");

    // cycle
    RemoveComment(aComment);
    AddComment(aComment, false);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::CommentRemoved(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentRemoved");

    RemoveComment(aComment);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::CommentAdded(WPComment aComment)
{
    qDebug("WPCommentGraphicsList::CommentAdded");

    AddComment(aComment, false);

    // update labels
    SetLabelText();
}

void WPCommentGraphicsList::SetLabelText()
{
    int height = 0;

    if (iUnapproved) {
        iUnapprovedLabel->setText(tr("Unapproved") + " (" + QString::number(iUnapproved) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iApproved) {
        iApprovedLabel->setText(tr("Comments") + " (" + QString::number(iApproved) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iApproved + iUnapproved) * (CommentMinimumHeight + KItemSpacing); 

    // update container height
    setMinimumHeight(iLayout->sizeHint(Qt::PreferredSize).height());
    
}

void WPCommentGraphicsList::DecrementUnapproved()
{
    if (--iUnapproved == 0) {
        iItems.removeAt(0);
        iLayout->removeItem(iUnapprovedProxy);
    }
}

void WPCommentGraphicsList::DecrementApproved()
{
    if (--iApproved == 0) {
        // start of list, skipping possible label + unapproved comments
        int index = iUnapproved ? 1 + iUnapproved : 0;

        iItems.removeAt(index);
        iLayout->removeItem(iApprovedProxy);
    }
}

void WPCommentGraphicsList::IncrementUnapproved()
{
    if (++iUnapproved == 1) {
        iLayout->insertItem(0, iUnapprovedProxy);
        iItems.insert(0, NULL); // placeholder
    }
}

void WPCommentGraphicsList::IncrementApproved()
{
    if (++iApproved == 1) {
        // start of list, skipping possible label + unapproved comments
        int index = iUnapproved ? 1 + iUnapproved : 0;

        iLayout->insertItem(index, iApprovedProxy);
        iItems.insert(index, NULL);
    }
}
