#include "wpgraphicslist.h"

#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsSceneResizeEvent>
#include <QPropertyAnimation>

#include "wpgraphicslistitem.h"
#include "wpscrollarea.h"
#include "PopupMenu.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "BlogView.h"
#include "wpkeydispatcher.h"
#include "wpcommongeometry.h"
    
const int WPGraphicsList::KItemSpacing = 5;

#ifdef Q_WS_MAEMO_5
const int WPGraphicsList::KTitleDefaultHeight = 45;
#else
const int WPGraphicsList::KTitleDefaultHeight = 32;
#endif

WPGraphicsList::WPGraphicsList(BlogView* aBlogView, const QGraphicsView& aGraphicsView,
	        PannableView* aPannableView)
  : iCurrentItem(NULL)
  , iBusyIndicator(aBlogView->GetBusyIndicator())
  , iThemeData(aBlogView->GetThemeData())
  , iGraphicsView(aGraphicsView) 
  , iKeyDispatcher(aBlogView)
  , iPopupMenu(NULL)
#ifndef Q_WS_MAEMO_5
  , iAnimation(NULL)
  , iMenuHiding(false)
#endif /* Q_WS_MAEMO_5 */
  , iPannableView(aPannableView)
{
	iLayout = new QGraphicsLinearLayout(Qt::Vertical);
	iLayout->setSpacing(KItemSpacing);
	iLayout->setContentsMargins(5, 0, 5, 0);
    setContentsMargins(0, 0, 0, 0);
    //iLayout->addStretch();
    setLayout(iLayout);
}

void WPGraphicsList::ShowContextMenu(WPGraphicsListItem* aItem, QPoint /*aPos*/)
{
    if (iPopupMenu)
        return;

    if (aItem->IsBusy() || aItem->BogusFocus())
        return;

    iPopupMenu = new PopupMenu;
    connect(iPopupMenu, SIGNAL(aboutToHide()), this, SLOT(MenuHiding()));
    iCurrentItem = aItem;
    iCurrentItem->MarkBogusFocus(true);
    
    PopulatePopupMenuForCurrentItem(iPopupMenu);
    
    iPopupMenu->setWindowOpacity(0.6);
    iPopupMenu->show();

    //QPoint itemPoint = iGraphicsView.mapFromScene(aItem->TopRight());
    //itemPoint = getMenuLocation(itemPoint);
    QPoint viewPoint = iGraphicsView.mapFromScene(aItem->scenePos());
    
    int x = viewPoint.x() + iGraphicsView.viewport()->width();
    int y = viewPoint.y();
    
    // fix out of screen
    //
    if (y + iPopupMenu->height() > iGraphicsView.viewport()->height())
    {
        y -= y + iPopupMenu->height() - iGraphicsView.viewport()->height();
    }
    if (y < 0) { y = 0; }

    // destination point to place popup menu at
    QRect dest(x - iPopupMenu->width(), y, iPopupMenu->width(), iPopupMenu->height());

// no animations on maemo    
#ifdef Q_WS_MAEMO_5
    iPopupMenu->setGeometry(dest);

#else    
    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x - iPopupMenu->width(), y);

    iAnimation->setStartValue(dest.adjusted(iPopupMenu->width(), 0, 0, 0));
    iAnimation->setEndValue(dest);
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();

#endif /* Q_WS_MAEMO_5 */
}

void WPGraphicsList::ContextMenuHidden()
{
#ifndef Q_WS_MAEMO_5
    iMenuHiding = false;
    delete iAnimation;
    iAnimation = NULL;
#endif

    iPopupMenu->deleteLater();
    iPopupMenu = NULL;

    setFocus();
}

void WPGraphicsList::MenuHiding()
{
#ifdef Q_WS_MAEMO_5
    // direct
    ContextMenuHidden();

#else    
    // never take this event twice.
    qDebug("WP::WPGraphicsList::MenuHiding()");
    if (iMenuHiding) return;
    
    if (iAnimation)
    {
        delete iAnimation;
        iAnimation = NULL;
    }

    if(!iPopupMenu) { return; }
    
    iPopupMenu->show();

    iAnimation = new QPropertyAnimation(iPopupMenu, "geometry", this);
    iAnimation->setDuration(200);
    
    QRectF itemGeometry = iPopupMenu->geometry();
    
    int x = int(itemGeometry.x());
    int y = int(itemGeometry.y());

    qDebug("WP::Animating iPopupMenu FROM %d, %d TO %d, %d", x, y, x + iPopupMenu->width(), y);

    iAnimation->setStartValue(QRect(x,y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEndValue(QRect(x+iPopupMenu->width(),y,iPopupMenu->width(),iPopupMenu->height()));
    iAnimation->setEasingCurve(QEasingCurve::Linear);
    iAnimation->start();

    iMenuHiding = true;
    connect(iAnimation,SIGNAL(finished()), this, SLOT(ContextMenuHidden()));
#endif    
}

void WPGraphicsList::setFocus()
{
    if (iCurrentItem && iItems.indexOf(iCurrentItem) >= 0)
    {
        iCurrentItem->setFocus();
        iCurrentItem->MarkBogusFocus(false); // gained real focus, disable bogus
    }
    else if (iItems.count() > FirstFocusableItemIndex())
    {
        iItems[FirstFocusableItemIndex()]->setFocus();
    }
    else
    {
        QGraphicsWidget::setFocus();
    }
}

void WPGraphicsList::resizeEvent(QGraphicsSceneResizeEvent* event)
{
    // XXX: without iCurrentItem?
    if (iPopupMenu && iPopupMenu->isVisible() && iCurrentItem)
        iPopupMenu->move(getMenuLocation(iCurrentItem->TopRight().toPoint()));

    iLayout->setGeometry(QRectF(0, 0, event->newSize().width(), event->newSize().height()));
    setMinimumHeight(iLayout->sizeHint(Qt::PreferredSize).height());
}

QPoint WPGraphicsList::getMenuLocation(QPoint itemPoint)
{
    const int xCorrection = itemPoint.x() + iPopupMenu->sizeHint().width() - iGraphicsView.viewport()->size().width();
    itemPoint += QPoint(-xCorrection, 0);
    return itemPoint;
}

void WPGraphicsList::keyPressEvent ( QKeyEvent * event  )
{
    qDebug("WP::WPGraphicsList::keyPressEvent Key event %d", event->key());
    if (event->key() == Qt::Key_Down)
    {
        // XXX: with NULL iCurrentItem?
        int i = iItems.indexOf(iCurrentItem);
        if (++i < iItems.count())
        {
            if (!iItems.at(i)) i++; // jump over the null -item.
            
            if (i < iItems.count() && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
    else if (event->key() == Qt::Key_Up)
    {
        int i = iItems.indexOf(iCurrentItem);
        if (--i >= 0)
        {
            if (!iItems.at(i)) i--; // jump over the null -item.
            
            if (i >= 0 && iItems.at(i))
            {
                iCurrentItem = iItems.at(i);
                QPointF point(iCurrentItem->scenePos());
                iPannableView->moveToItem(iCurrentItem);
                setFocus();
            }
        }
        event->accept();
    }
}

void WPGraphicsList::keyReleaseEvent ( QKeyEvent * event  )
{
    if (event->key() != Qt::Key_Down &&
        event->key() != Qt::Key_Up )
    {
        iKeyDispatcher->keyClickedEvent(event);
    }
}
