#include "wpgraphicslistitem.h"
#include "busyindicator.h"
#include "themedatastorage.h"

#include <QGraphicsSceneMouseEvent>
#include <QPoint>
#include <QRect>
#include <QKeyEvent>
#include <QPainter>

static const int KPostDefaultHeight = 57;

WPGraphicsListItem::WPGraphicsListItem(BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : iKeyPressReceived(false)
    , iWidth(0.0)
    , iHeight(0.0)
    , iXMargin(0.0)
    , iYMargin(0.0)
    , iBusyX(0)
    , iBusyY(0)
    , iPicWidth(0)
    , iPicHeight(0)
    , iCol1ContentXOffset(0)
    , iTimerId(0)
    , iBusyIndicator(aBusyIndicator)
    , iHasMenuFocus(false)
    , iThemeData(aThemeData)
{
    /* Size can not be set here, as the virtual table is not initialized,
     * and it would end up in the resize function
     */
    setFocusPolicy(Qt::StrongFocus);
}

void WPGraphicsListItem::mousePressEvent(QGraphicsSceneMouseEvent *e)
{
    QGraphicsWidget::mousePressEvent(e);
    e->accept();
}

void WPGraphicsListItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *e)
{
    if (hasFocus())
    {
        emit Selected(this, e->screenPos());
    }
    QGraphicsWidget::mouseReleaseEvent(e);
}

void WPGraphicsListItem::keyPressEvent(QKeyEvent *e)
{
    if (e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter)
    {
        // Check later if we got the full click, or just the release event.
        // If an user clicks a softkey menu, the press is consumed but
        // release is not -- we do not want partial event to trigger the
        // popup menu.
        iKeyPressReceived = true;
    }
    else
    {
        QGraphicsWidget::keyPressEvent(e);
    }
}

void WPGraphicsListItem::keyReleaseEvent(QKeyEvent *e)
{
    if((e->key() == Qt::Key_Select || e->key() == Qt::Key_Enter) && iKeyPressReceived)
    {
        iKeyPressReceived = false;
        emit Selected(this, TopRight().toPoint()); // TODO: Fix the correct coordinates
    }
    else
    {
        QGraphicsWidget::keyReleaseEvent(e);
    }
}
#include <QGraphicsLayout>

void WPGraphicsListItem::resizeEvent(QGraphicsSceneResizeEvent *e)
{
    prepareGeometryChange();
    iWidth = e->newSize().width();
    iHeight = e->newSize().height();
    
    // Busy indicator
    const QPixmap& px = iBusyIndicator.PixmapToDraw();
    iBusyX = (int)iWidth/2 - px.width()/2;
    iBusyY = (int)iHeight/2 - px.height()/2;
    
    iCol1ContentXOffset = int((iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width()) + 5;
    
    // Calculate the bounding rect and take into account the shadow margin in the original picture.
    iXMargin = (iThemeData.iListItemShadowMargin * iWidth)/iThemeData.iListItemPixmap->width();
    iYMargin = (iThemeData.iListItemShadowMargin * iHeight)/iThemeData.iListItemPixmap->height();
	iBoundingRect.setRect(iXMargin, iYMargin, iWidth - (2*iXMargin), iHeight - (2*iYMargin));
	
    iPicWidth = int(iBoundingRect.width() + (2*iXMargin));
    iPicHeight = int(iBoundingRect.height() + (2*iYMargin));
    
    handleGeomeryChange();
    
    updateGeometry();
}

QPointF WPGraphicsListItem::TopRight() const
{
    return geometry().topRight();
}

QRectF WPGraphicsListItem::boundingRect() const
{
    return iBoundingRect;
}

void WPGraphicsListItem::MarkBogusFocus(bool aActive)
{
    iHasMenuFocus = aActive;
}

bool WPGraphicsListItem::BogusFocus()
{
    return iHasMenuFocus;
}

void WPGraphicsListItem::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
    painter->setPen(Qt::NoPen);
    painter->drawPixmap(0,0,iPicWidth,iPicHeight,*iThemeData.iListItemPixmap);

    if (hasFocus() || iHasMenuFocus)
    {
        QColor color = QColor(247,247,247);
        painter->setBrush(color);
        painter->drawRect(boundingRect());
    }

    // Deriving class stuff
    paintContent(painter, option, widget);

    // States
    if (iTimerId != 0)
    {
        killTimer(iTimerId);
        iTimerId = 0;
    }

    if (IsBusy())
    {
        painter->setBrush(QColor(0, 0, 0, 128));
        painter->drawRect(boundingRect());
        painter->setBrush(QColor(0,0,0));
        painter->drawPixmap(QPoint(iBusyX,iBusyY), iBusyIndicator.PixmapToDraw());
        iTimerId = startTimer(300);
    }
    else if (HasError())
    {
        QFont errorFont("Arial");
        errorFont.setBold(true);
        errorFont.setPixelSize(24);
        painter->setFont(errorFont);
        painter->setPen(Qt::red);
        painter->drawText(int(boundingRect().topRight().x()-25),int(boundingRect().topRight().y()+30),QString("!"));
    }
}

void WPGraphicsListItem::timerEvent(QTimerEvent*)
{
    update();
}
