
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneResizeEvent>
#include <QLabel>
#include <QPropertyAnimation>
#include <QPushButton>

#include "wppagegraphicslistitem.h"
#include "wppagegraphicslist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "PagesView.h"
#include "wpscrollarea.h"
#include "BlogView.h"
#include "PopupMenu.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "wpkeydispatcher.h"
#include "wpconfirmdialog.h"

WPPageGraphicsList::WPPageGraphicsList(QListIterator<WPPage> aPages,
        const QGraphicsView& aGraphicsView,BlogView* aBlogView, PannableView* aPannableView
    ) : 
        WPGraphicsList(aBlogView, aGraphicsView, aPannableView), 
        iDrafts(0), iPages(0)
{
    // TODO find a proper way to align menu icon and text side by side.
    iActionOpen = new QAction(iThemeData.iContextMenuIcons[ThemeData::EOpen],tr("    Open"), this);
    iActionEdit = new QAction(iThemeData.iContextMenuIcons[ThemeData::EEdit],tr("    Edit"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);

    connect(iActionOpen, SIGNAL(triggered(bool)), this, SLOT(Open()));
    connect(iActionEdit, SIGNAL(triggered(bool)), this, SLOT(Edit()));
    connect(iActionDelete, SIGNAL(triggered(bool)), this, SLOT(Delete()));

    iNewPageButton = new QPushButton;
    iNewPageButton->setText(tr("Write new page"));
    iNewPageProxy = new QGraphicsProxyWidget(this);
    iNewPageProxy->setWidget(iNewPageButton);
    iNewPageButton->setObjectName(QString::fromLatin1("newPageLabel"));
    //iNewPageButton->setFixedHeight(KTitleDefaultHeight);
    //iNewPageButton->setStyleSheet(QLatin1String(KButtonStyleSheet));
    connect(iNewPageButton, SIGNAL(clicked(bool)), aBlogView, SLOT(ShowWriteNewPageView()));

    iLayout->addItem(iNewPageProxy);
    iItems.append(NULL);    // placeholder

    // labels, not automatically added 
    iDraftsLabel = new QLabel;
    iDraftsProxy = new QGraphicsProxyWidget(this);
    iDraftsProxy->setWidget(iDraftsLabel);
    iDraftsLabel->setObjectName(QString::fromLatin1("draftsLabel"));
    iDraftsLabel->setFixedHeight(KTitleDefaultHeight);
    iDraftsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iPagesLabel = new QLabel;
    iPagesProxy = new QGraphicsProxyWidget(this);
    iPagesProxy->setWidget(iPagesLabel);
    iPagesLabel->setObjectName(QString::fromLatin1("pagesLabel"));
    iPagesLabel->setFixedHeight(KTitleDefaultHeight);
    iPagesLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));
 
    while (aPages.hasNext()) {
        WPPage page = aPages.next();

        AddPage(page);
    }
    
    // update label text, and container height
    SetLabelText();
}

WPPageGraphicsList::~WPPageGraphicsList()
{

}

void WPPageGraphicsList::PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu)
{
	aPopupMenu->addAction(iActionOpen);
	aPopupMenu->addAction(iActionEdit);
	aPopupMenu->addAction(iActionDelete);
}

void WPPageGraphicsList::AddPage(WPPage aPage)
{
    QStringList list;

    if (aPage->IsLocal())
        list << "Local draft";

    list << WPNetworkEngine::LocalizedTime(aPage->iDateCreated);

    WPPageGraphicsListItem* item = new WPPageGraphicsListItem(aPage, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(Selected(WPGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPGraphicsListItem*, QPoint)));
    
    // update counters/labels
    if (aPage->IsDraft())
        IncrementDrafts();
    else
        IncrementPages();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aPage->IsDraft() && index > 1)
                // draft item at end of draft list, currently on the items label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPPage list_page = ItemAt(index)->GetPage();
        
        // drafts go first
        if (aPage->IsDraft() && !list_page->IsDraft())
            // XXX: never run, because of NULL index above
            break;

        if (!aPage->IsDraft() && list_page->IsDraft())
            // do not mix published in with drafts
            continue;

        // find first older post
        if (list_page->GetLocalTime() < aPage->GetLocalTime())
            break;
    }
    
    // insert before older post
    iItems.insert(index, item);

    // skip "New page" button
    iLayout->insertItem(index, item);
}

void WPPageGraphicsList::RemovePage (WPPage aPage)
{
    int index;
    WPPageGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = ItemAt(index);

        if (item && item->GetPage() == aPage)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPPage->IsDraftOrPending(), because it might have changed
    bool isDraft = index - 1 <= iDrafts;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isDraft)
        DecrementDrafts();
    else
        DecrementPages();

}

void WPPageGraphicsList::Open()
{
    qDebug("WP::WPPageGraphicsList::Open");
    emit OpenClicked(CurrentItem()->GetPage());
}

void WPPageGraphicsList::Edit()
{
    qDebug("WP::WPPageGraphicsList::Edit");
    emit EditClicked(CurrentItem()->GetPage());
}

void WPPageGraphicsList::Delete()
{
    WP_CONFRIM_AND_RETURN_ON_FALSE

    qDebug("WP::WPPageGraphicsList::Delete");
    CurrentItem()->GetPage()->Delete();
}

void WPPageGraphicsList::PageChanged(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageChanged");

    // cycle
    RemovePage(aPage);
    AddPage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::PageRemoved(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageRemoved");

    RemovePage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::PageAdded(WPPage aPage)
{
    qDebug("WPPageGraphicsList::PageAdded");

    AddPage(aPage);
    
    // update labels
    SetLabelText();
}

void WPPageGraphicsList::SetLabelText()
{
    int height = 0;
    
    height += iNewPageButton->height() + KItemSpacing;

    if (iDrafts) {
        iDraftsLabel->setText(tr("Drafts") + " (" + QString::number(iDrafts) + ')');

        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iPages) {
        iPagesLabel->setText(tr("Pages") + " (" + QString::number(iPages) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iDrafts + iPages) * (WPPageGraphicsListItem::ItemMinimumHeight + KItemSpacing); 
   
    // update container height
    setMinimumHeight(height);
}

void WPPageGraphicsList::IncrementDrafts()
{
    if (++iDrafts == 1) {
        iLayout->insertItem(1, iDraftsProxy);
        iItems.insert(1, NULL); // placeholder
    }
}

void WPPageGraphicsList::DecrementDrafts()
{
    if (--iDrafts == 0) {
        iItems.removeAt(1);
        iLayout->removeItem(iDraftsProxy);
    }
}

void WPPageGraphicsList::DecrementPages()
{
    if (--iPages == 0) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iItems.removeAt(index);
        iLayout->removeItem(iPagesProxy);
    }
}

void WPPageGraphicsList::IncrementPages()
{
    if (++iPages == 1) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iLayout->insertItem(index, iPagesProxy);
        iItems.insert(index, NULL);
    }
}

