
#include <QGraphicsLinearLayout>
#include <QGraphicsView>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneResizeEvent>
#include <QLabel>
#include <QPropertyAnimation>
#include <QPushButton>

#include "wppostgraphicslistitem.h"
#include "wppostgraphicslist.h"
#include "WPData.h"
#include "WPNetworkEngine.h"
#include "PostsView.h"
#include "wpscrollarea.h"
#include "BlogView.h"
#include "PopupMenu.h"
#include "themedatastorage.h"
#include "enume/pannableview.h"
#include "wpkeydispatcher.h"
#include "wpconfirmdialog.h"

WPPostGraphicsList::WPPostGraphicsList(QListIterator<WPPost> aPosts,
        const QGraphicsView& aGraphicsView,BlogView* aBlogView, PannableView* aPannableView
    ) : 
		WPGraphicsList(aBlogView, aGraphicsView, aPannableView),
        iDrafts(0), iPosts(0)
{   
    // TODO find a proper way to align menu icon and text side by side.
    iActionOpen = new QAction(iThemeData.iContextMenuIcons[ThemeData::EOpen],tr("    Open"), this);
    iActionEdit = new QAction(iThemeData.iContextMenuIcons[ThemeData::EEdit],tr("    Edit"), this);
    iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);

    connect(iActionOpen, SIGNAL(triggered(bool)), this, SLOT(Open()));
    connect(iActionEdit, SIGNAL(triggered(bool)), this, SLOT(Edit()));
    connect(iActionDelete, SIGNAL(triggered(bool)), this, SLOT(Delete()));

    iNewPostButton = new QPushButton;
    iNewPostButton->setText(tr("Write new post"));
    iNewPostProxy = new QGraphicsProxyWidget(this);
    iNewPostProxy->setWidget(iNewPostButton);
    iNewPostButton->setObjectName(QString::fromLatin1("newPostLabel"));
    //iNewPostButton->setFixedHeight(KTitleDefaultHeight);
    //iNewPostButton->setStyleSheet(QLatin1String(KButtonStyleSheet));
    connect(iNewPostButton, SIGNAL(clicked(bool)), aBlogView, SLOT(ShowWriteNewPostView()));

    iLayout->addItem(iNewPostProxy);
    iItems.append(NULL);    // placeholder

    // labels, not automatically added 
    iDraftsLabel = new QLabel;
    iDraftsProxy = new QGraphicsProxyWidget(this);
    iDraftsProxy->setWidget(iDraftsLabel);
    iDraftsLabel->setObjectName(QString::fromLatin1("draftsLabel"));
    iDraftsLabel->setFixedHeight(KTitleDefaultHeight);
    iDraftsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));

    iPostsLabel = new QLabel;
    iPostsProxy = new QGraphicsProxyWidget(this);
    iPostsProxy->setWidget(iPostsLabel);
    iPostsLabel->setObjectName(QString::fromLatin1("postsLabel"));
    iPostsLabel->setFixedHeight(KTitleDefaultHeight);
    iPostsLabel->setStyleSheet(QLatin1String(KLabelStyleSheet));
   
    while (aPosts.hasNext()) {
        WPPost post = aPosts.next();

        AddPost(post);
    }
    
    // update label text, and container height
    SetLabelText();
}

WPPostGraphicsList::~WPPostGraphicsList()
{

}


void WPPostGraphicsList::PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu)
{
    aPopupMenu->addAction(iActionOpen);
    aPopupMenu->addAction(iActionEdit);
    aPopupMenu->addAction(iActionDelete);
}

void WPPostGraphicsList::AddPost(WPPost aPost)
{
    QStringList list;

    if (aPost->IsLocal())
        list << "Local draft";

    list << WPNetworkEngine::LocalizedTime(aPost->iDateCreated);

    WPPostGraphicsListItem* item = new WPPostGraphicsListItem(aPost, iBusyIndicator, iThemeData);
    connect(item, SIGNAL(Selected(WPGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPGraphicsListItem*, QPoint)));

    // update counters/labels
    if (aPost->IsDraft())
        IncrementDrafts();
    else
        IncrementPosts();

    // insert by increasing age
    int index;

    for (index = 0; index < iItems.size(); index++) {
        if (iItems[index] == NULL) {
            if (aPost->IsDraft() && index > 1)
                // draft item at end of draft list, currently on the items label
                break;

            else
                // placeholder for label, skip
                continue;
        }

        WPPost list_post = ItemAt(index)->GetPost();
        
        // drafts go first
        if (aPost->IsDraft() && !list_post->IsDraft())
            // XXX: never run, because of NULL index above
            break;

        if (!aPost->IsDraft() && list_post->IsDraft())
            // do not mix published in with drafts
            continue;

        // find first older post
        if (list_post->GetLocalTime() < aPost->GetLocalTime())
            break;
    }
    
    // insert before older post
    iItems.insert(index, item);
    iLayout->insertItem(index, item);
}

void WPPostGraphicsList::RemovePost (WPPost aPost)
{
    int index;
    WPPostGraphicsListItem* item = NULL;

    // find it
    for (index = 0; index < iItems.size(); index++) {
        item = ItemAt(index);

        if (item && item->GetPost() == aPost)
            // found
            break;

        // sentinel
        item = NULL;
    }

    if (!item)
        // not found
        return;

    // can't use WPPost->IsDraftOrPending(), because it might have changed
    bool isDraft = index - 1 <= iDrafts;

    // drop from list
    iLayout->removeItem(item);
    iItems.removeAt(index);

    // reset current item
    if (iCurrentItem == item)
        iCurrentItem = NULL;

    item->deleteLater();

    // update count
    if (isDraft)
        DecrementDrafts();
    else
        DecrementPosts();
}

void WPPostGraphicsList::Open()
{
    qDebug("WP::WPPostGraphicsList::Open");
    emit OpenClicked(CurrentItem()->GetPost());
}

void WPPostGraphicsList::Edit()
{
    qDebug("WP::WPPostGraphicsList::Edit");
    emit EditClicked(CurrentItem()->GetPost());
}

void WPPostGraphicsList::Delete()
{
    WP_CONFRIM_AND_RETURN_ON_FALSE

    qDebug("WP::WPPostGraphicsList::Delete");
    emit DeleteClicked(CurrentItem()->GetPost());
}

void WPPostGraphicsList::PostChanged(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostChanged");

    // cycle
    RemovePost(aPost);
    AddPost(aPost);
    
    // update labels
    SetLabelText();
}

void WPPostGraphicsList::PostRemoved(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostRemoved");

    RemovePost(aPost);

    // update labels
    SetLabelText();
}

void WPPostGraphicsList::PostAdded(WPPost aPost)
{
    qDebug("WPPostGraphicsList::PostAdded");

    AddPost(aPost);

    // update labels
    SetLabelText();
}

void WPPostGraphicsList::SetLabelText()
{
    int height = 0;
    
    height += iNewPostButton->height() + KItemSpacing;

    if (iDrafts) {
        iDraftsLabel->setText(tr("Drafts") + " (" + QString::number(iDrafts) + ')');

        height += KTitleDefaultHeight + KItemSpacing;
    }

    if (iPosts) {
        iPostsLabel->setText(tr("Posts") + " (" + QString::number(iPosts) + ')');
        
        height += KTitleDefaultHeight + KItemSpacing;
    }

    height += (iDrafts + iPosts) * (WPPostGraphicsListItem::ItemMinimumHeight + KItemSpacing); 
   
    // update container height
    setMinimumHeight(height);
}

void WPPostGraphicsList::IncrementDrafts()
{
    if (++iDrafts == 1) {
        iLayout->insertItem(1, iDraftsProxy);
        iItems.insert(1, NULL); // placeholder
    }
}

void WPPostGraphicsList::DecrementDrafts()
{
    if (--iDrafts == 0) {
        iItems.removeAt(1);
        iLayout->removeItem(iDraftsProxy);
    }
}

void WPPostGraphicsList::DecrementPosts()
{
    if (--iPosts == 0) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iItems.removeAt(index);
        iLayout->removeItem(iPostsProxy);
    }
}

void WPPostGraphicsList::IncrementPosts()
{
    if (++iPosts == 1) {
        // start of list, skipping button + possible label&unapproved comments
        int index = iDrafts ? 1 + 1 + iDrafts : 1;

        iLayout->insertItem(index, iPostsProxy);
        iItems.insert(index, NULL);
    }
}

